@@ -8,11 +8,13 @@ import sys
 import argparse
 import logging
 from urllib.parse import urlparse
-from pyVmomi import vim
+from pyVmomi import vim, vmodl
 from pyVim.connect import Disconnect, SmartConnectNoSSL
 from infi.pyvmomi_wrapper.esxcli import EsxCLI
+from zabbix_api import ZabbixAPIException
 
 parser = argparse.ArgumentParser("Correct discoverd host name")
+parser.add_argument('-l', '--limit', action='append', type=str, help="Specify hypervisors")
 parser.set_defaults(handler=lambda args: main(args))
 
 
@@ -55,7 +57,7 @@ def fetch_esxi(host, user, pwd, esxi_name, port=443):
                     isFind = True
 
             if isFind:
-                # location 默认值为datacenter的名字 
+                # location 默认值为datacenter的名字
                 esxi_info['location'] = view.name
                 # 如果datacenter有“机房名称”自定义属性，则覆盖前者。
                 for prop in view.customValue:
@@ -63,6 +65,7 @@ def fetch_esxi(host, user, pwd, esxi_name, port=443):
                         esxi_info['location'] = prop.value
                         break
 
+
     # 取esxi属性
     for esxi_ in container.view:
         if esxi_.name != esxi_name:
@@ -81,16 +84,19 @@ def fetch_esxi(host, user, pwd, esxi_name, port=443):
                            esxi_.summary.hardware.numCpuCores,
                            esxi_.summary.hardware.numCpuThreads)
         esxi_info['mem'] = f'内存: {esxi_.summary.hardware.memorySize/1024/1024//1024}GB'
+
         # get host's ipA and netmask
         for vnic in esxi_.config.network.vnic:
             if isinstance(vnic, vim.host.VirtualNic):
                 esxi_info['ip'] = vnic.spec.ip.ipAddress
                 esxi_info['subnet_mask'] = vnic.spec.ip.subnetMask
+
         # get host's mac address
         for portgroup in esxi_.config.network.portgroup:
             for port in portgroup.port:
                 if port.type == 'host':
                     esxi_info['mac_address'] = ''.join(port.mac)
+
         # get host's serial_number
         ordered_keys = ['SerialNumberTag', 'EnclosureSerialNumberTag', 'ServiceTag']
         for iden in esxi_.summary.hardware.otherIdentifyingInfo:
@@ -134,52 +140,64 @@ def main(args):
     if not zbx_group:
         sys.exit()
 
-    hosts = zbx_group[0].get('hosts')
-    macros = {}
+    grp_hosts = zbx_group[0].get('hosts')
 
-    tmp_ = zapi.host.get({
+    # 如指定limit参数, 则仅处理列表中的host 
+    if args.limit:
+        grp_hosts = [h for h in grp_hosts if h['name'] in args.limit]
+
+
+    # 调用zapi 查询host的macros信息
+    hosts = zapi.host.get({
         'output': ['hostid', 'name'],
-        'hostids': [h.get('hostid') for h in hosts],
-        'selectMacros': 'extend'
+        'hostids': [h['hostid'] for h in grp_hosts],
+        'selectMacros': ['macro', 'value']
     })
-    for host_ in tmp_:
-        macros[host_['name']] = {}
-        for ma_ in host_.get('macros'):
-            macros[host_['name']][ma_.get('macro')] = ma_.get('value')
 
     for host in hosts:
-        host_id = host.get('hostid')
-        host_name = host.get('name')
-        vcenter_ip = urlparse(macros[host_name].get('{$URL}')).hostname
-        vcenter_admin = macros[host_name].get('{$USERNAME}')
-        vcenter_pwd = macros[host_name].get('{$PASSWORD}')
+        host_id = host['hostid']
+        host_name = host['name']
+
+        # 从zabbix host的macros 中取出对应的vcenter登录信息
+        url = ''.join([ma['value'] for ma in host['macros'] if ma['macro'] == '{$URL}'])
+        vcenter_ip = urlparse(url).hostname
+        vcenter_admin = ''.join([ma['value'] for ma in host['macros'] if ma['macro'] == '{$USERNAME}'])
+        vcenter_pwd = ''.join([ma['value'] for ma in host['macros'] if ma['macro'] == '{$PASSWORD}'])
 
         logging.debug(f'[fetch_esxi] vcenter_ip: {vcenter_ip}, esxi_name:{host_name}')
-        esxi_info = fetch_esxi(host=vcenter_ip,
-                               user=vcenter_admin,
-                               pwd=vcenter_pwd,
-                               esxi_name=host_name)
 
-        zapi.host.update({
-            "hostid": host_id,
-            "inventory": {
-                "type": esxi_info.get('type'),
-                "name": esxi_info.get('name'),
-                "alias": esxi_info.get('ip'),
-                "os": esxi_info.get('os'),
-                "os_full": esxi_info.get('os_full'),
-                "os_short": esxi_info.get('os_short'),
-                "serialno_a": esxi_info.get('serial_number'),
-                "tag": esxi_info.get('tag'),
-                "macaddress_a": esxi_info.get('mac_address'),
-                "macaddress_b": esxi_info.get('mac_address'),
-                "model": esxi_info.get('model'),
-                "vendor": esxi_info.get('vendor'),
-                "host_networks": esxi_info.get('ip'),
-                "host_netmask": esxi_info.get('subnet_mask'),
-                "hardware_full": '\n'.join([esxi_info.get('cpu',''), esxi_info.get('mem','')]),
-                "location": esxi_info.get('location'),
-                "site_rack": esxi_info.get('site_rack')
-            }
-        })
-        logging.info(f'update host->{host_name} success!')
+        try:
+            esxi_info = fetch_esxi(host=vcenter_ip,
+                                   user=vcenter_admin,
+                                   pwd=vcenter_pwd,
+                                   esxi_name=host_name)
+
+            zapi.host.update({
+                "hostid": host_id,
+                "inventory": {
+                    "type": esxi_info.get('type'),
+                    "name": esxi_info.get('name'),
+                    "alias": esxi_info.get('ip'),
+                    "os": esxi_info.get('os'),
+                    "os_full": esxi_info.get('os_full'),
+                    "os_short": esxi_info.get('os_short'),
+                    "serialno_a": esxi_info.get('serial_number'),
+                    "tag": esxi_info.get('tag'),
+                    "macaddress_a": esxi_info.get('mac_address'),
+                    "macaddress_b": esxi_info.get('mac_address'),
+                    "model": esxi_info.get('model'),
+                    "vendor": esxi_info.get('vendor'),
+                    "host_networks": esxi_info.get('ip'),
+                    "host_netmask": esxi_info.get('subnet_mask'),
+                    "hardware_full": '\n'.join([esxi_info.get('cpu',''), esxi_info.get('mem','')]),
+                    "location": esxi_info.get('location'),
+                    "site_rack": esxi_info.get('site_rack')
+                }
+            })
+            logging.info(f'update host->{host_name} success!')
+
+        except ZabbixAPIException as error:
+            logging.exception(error)
+        except vmodl.MethodFault as error:
+            logging.exception(error.msg)
+            continue
